//  Initial code taken from "Constructing Intelligent Agents with Java" by
//  Bigus & Bigus - with extensive modifications

//  Modified by:  Paul Marlow, Amir Ghavam, Yoga Selvaraj
//  Date Due:     November 30, 2000

import java.util.*;
import java.io.*;
import java.awt.*;

// A DataSet is read from a text file
// The variables must be defined and added in the correct order
// The string data is available in the data member
// The normalized numeric data is available in the normalizedData member
public class DataSet extends Object
{
  String fileName ;

  Vector data ;             // raw data from file
  Vector normalizedData;

  int fieldsPerRec=0;
  int numRecords=0 ;
  int numInputs=0;
  int numOutputs=0;
  int numHidden=0;

  static public void trace(String text) { System.out.println(text);}

  // Constructor
  public DataSet(String FileName)
  {
    fileName = FileName ;     // text file name
    fieldsPerRec = 0 ;        // start with no variables defined
    data = new Vector() ;     // holds string data
  }

  // Reads in data from the file supplied via the constructor
  public void loadDataFile()
  {
    String tempRec[]=null ;   // used when data is symbolic

    String line=null ;
    BufferedReader in=null ;
    try {
      in = new BufferedReader(new InputStreamReader(new FileInputStream(fileName)));
    }
    catch (FileNotFoundException exc) {
      trace("Error: Can't find file " + fileName + ".dat" ) ;
    }

    int recInx = 0 ;
    int token= 0 ;
    StringTokenizer input = null ;
    try {
      line = in.readLine();
      fieldsPerRec = new Integer(line).intValue();
      line = in.readLine();
      numInputs = new Integer(line).intValue();
      line = in.readLine();
      numHidden = new Integer(line).intValue();
      line = in.readLine();
      numOutputs = new Integer(line).intValue();
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    do {
      try {
        line = in.readLine();
        if (line != null) {
          input = new StringTokenizer(line) ;
          tempRec = new String[fieldsPerRec] ;
          data.addElement(tempRec) ; // add record
         }
         else {
            break ;
         }
      } catch (IOException exc) {
        trace("Error reading file: " + fileName) ;
      }

      for(int i= 0 ; i < fieldsPerRec ; i++) {
         tempRec[i] = input.nextToken() ;
      }
      recInx++ ;
    } while (token != StreamTokenizer.TT_EOF) ;
    numRecords = recInx ;
    trace("\nLoaded " + numRecords + " records into memory.\n") ;
  }

  // Method which takes the raw data and normailzes it to use float values
  public void normalizeData()
  {
    String tempRec[]= null ;

    normalizedData = new Vector() ;
    Enumeration rawData = data.elements() ;
    while(rawData.hasMoreElements())
    {
      int inx = 0 ;
      float normNumRec[] = new float[fieldsPerRec] ;
      tempRec = (String[])rawData.nextElement() ;

      for (int i=0 ; i < fieldsPerRec ; i++)
      {
        normNumRec[i] = new Float(tempRec[i]).floatValue();
      }
      normalizedData.addElement(normNumRec) ;
    }
  }
}