//  Modified by:  Paul Marlow, Amir Ghavam, Yoga Selvaraj
//  Date Due:     November 30, 2000

// GoalieBrain class extends the Brain class functionality for goalie
// specific actions.

import java.io.*;

public class GoalieBrain extends Brain
{
  private boolean hasBall = false;
  private double currentDirection=0;
  private double dashedDistance=0;
  private FileOutputStream output;

  // Constructor
  public GoalieBrain(SendCommand IAS, String team, char side, int number,
          String playMode){
    super(IAS,team,side,number,playMode);

    // Create output file to aid in debugging
    try
    {
      output = new FileOutputStream("brainout.txt", true);
    }
    catch (FileNotFoundException e)
    {
      System.out.println("Error, file not found");
    }
  }

  // Method for writing a string to a file
  public void outToFile(String message)
  {
    String outputMessage = message + "\n";
    try
    {
      output.write(outputMessage.getBytes());
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  // Overwritten run method for goalie implementation
	public void run()
	{
    String activators;
		ObjectInfo object = null;
    currentDirection = 0;
    dashedDistance = 0;
		// first put it somewhere on my side
    m_IAS.move( -50 , 0);

    // Point player in direction of centre field
    object = m_memory.getObject("ball");

    // m_IAS.sensebody();

		while( !m_timeOver )
		{
      // Reset goalie's position in it's own goal
      if(m_resetPosition)
      {
        resetPlayerPosition();
        // m_IAS.sensebody();
      }

      // Find ball object
			object = m_memory.getObject("ball");
      if (object == null)
      {
        // see if object "Ball" exists
        object = m_memory.getObject("Ball");
      }

			if( object == null )
			{
				// If you don't know where is ball then find it
				m_IAS.turn(60);
        currentDirection+=60;
				m_memory.waitForNewInfo();
			}
      else  // We see the ball
      {
        if (object.m_distance <= 1.5) // KICK
        {
          double kickDirection = getKickDirection(currentDirection,dashedDistance);//(90.0 * Math.random()) - 45.0;
          /*
          activators = "KICK: (" + object.m_direction + " " + object.m_distance + " " +
                        currentDirection + " " + dashedDistance + ") " +
                        kickDirection + " 100.0";
          outToFile(activators);
          */

          m_IAS.kick(100,kickDirection);
          m_resetPosition=true;
        }
        //the ball is too far to catch
        else
        {
          if (object.m_direction != 0)  // TURN
          {
            /*
            activators = "TURN: (" + object.m_direction + " " + object.m_distance + " " +
                        currentDirection + " " + dashedDistance + ") " +
                        object.m_direction + " 0.0";
            outToFile(activators);
            */

            // An approximation of angle turned thus far for calculating
            // kick direction.
            currentDirection += object.m_direction;
            m_IAS.turn(object.m_direction);

            // Change angle from right (+) to left (-) or vice-versa to ensure
            // angle stays between [-180, 180]
            if (currentDirection > 180 || currentDirection < -180)
            {
              int reverseSign = (currentDirection > 0) ? -1 : 1;
              currentDirection = Math.abs(currentDirection);
              currentDirection = reverseSign * (180 - (currentDirection - 180));
            }
          }
          else  // DASH
          {
            // Only dash up to a certain distance, so goalie doesn't stray far
            // from the goal.
            if (object.m_distance <= 25.0)
            {
              double dashPower = 4*object.m_distance;
              /*
              activators = "DASH: (" + object.m_direction + " " + object.m_distance +
                        currentDirection + " " + dashedDistance + ") " +
                        " 0.0 " + dashPower;
              outToFile(activators);
              */

              m_IAS.dash(dashPower);
              m_hasMoved = true;
              if (m_hasMoved)
              {
                // An approximation used in the calculation of kick direction
                dashedDistance = object.m_distance;
              }
            }
          }
        }
      }

			// sleep one step to ensure that we will not send
			// two commands in one cycle.
			try{
				Thread.sleep(2*SoccerParams.simulator_step);
			}catch(Exception e){}
		}
	}

  // Examines 'hear' message from the referee to determine current play_mode
	public void hear(int time, String message)
	{
    if (message.startsWith("kick") ||
        message.startsWith("corner_kick") ||
        message.startsWith("offside") ||
        message.startsWith("goal") ||
        message.startsWith("foul") ||
        message.startsWith("free_kick") ||
        message.startsWith("time_up") ||
        message.startsWith("extend") ||
        message.startsWith("half_time"))
    {
      // Set reset flag if goalie has moved
      if(m_hasMoved)
        m_resetPosition = true;

      // outToFile(message);
    }
	}

  // Resets the goalie to a particular spot after he/she has moved from their
  // default position.
  public void resetPlayerPosition()
  {
		ObjectInfo object = null;
    String objectFlag = "goal " + m_side;

    // Try to find our own goal
    object = m_memory.getObject(objectFlag);

    // Find own goal, turn to it, and go to it.
    while (object == null || object.m_direction != 0 || object.m_distance > 1)
    {
      if (object == null) // Can't see goal, so turn
      {
        m_IAS.turn(90);
      }
      else if (object.m_direction != 0) // Can see goal, but need to turn more
      {
        m_IAS.turn(object.m_direction);
      }
      else  // Can see goal directly infront, so dash towards it
      {
        m_IAS.dash(4*object.m_distance);
      }

      m_memory.waitForNewInfo();
      object = m_memory.getObject(objectFlag);
    }

    // Here goalie is at own goal
    objectFlag = "flag c";
    object = m_memory.getObject(objectFlag);

    // Turn towards centre field
    while (object == null || object.m_direction != 0)
    {
      if (object == null) // Can't see centre field
      {
        m_IAS.turn(90);
      }
      else  // Turn to face centre field
      {
        m_IAS.turn(object.m_direction);
      }
      m_memory.waitForNewInfo();
      object = m_memory.getObject(objectFlag);
    }

    // Move out of goal slightly, so goalie isn't directly on the goal line
    m_IAS.dash(10);

    // Reset flags
    m_resetPosition = false;
    m_hasMoved = false;
    currentDirection = 0;
    dashedDistance = 0;

    m_memory.waitForNewInfo();
  }

  // Returns opponents side
  public char toggleSide()
  {
    if(m_side=='l')
      return 'r';
    else
      return 'l';
  }

  // Method to calculate approximately the direction of a kick, so the ball
  // will go down the field by use of trigonometry
  public double getKickDirection(double direction, double distance)
  {
    double top = distance*Math.sin(Math.toRadians(direction));
    double bottom = 100 - distance*Math.cos(Math.toRadians(direction));
    double arc = top/bottom;
    double angle = direction + Math.toDegrees(Math.atan(arc));
    return angle*(-1);
  }
}
