 
//	File:			Log.java
//	Author:		Paul Marlow
//	Date:			July 21, 2001
//

import java.util.*;
import java.io.*;

public class Log {
  private FileOutputStream output;
  private BufferedReader inFile;
  private int time = 0;
  private String log_message = "";
  private String filename = "";
  private boolean loaded = false;

  public Log()
  {
    filename = "log.txt";
    try
    {
      output = new FileOutputStream(filename, false);
    }
    catch (FileNotFoundException e)
    {
      System.out.println("Error, file not found");
    }
  }

  // ******** PAUL'S ADDITIONS ******** //

  public Log(String inputFile)
  {
    try {
      inFile = new BufferedReader(new InputStreamReader(new FileInputStream(inputFile)));
      filename = inputFile;
      loaded = true;
      try {
        log_message = inFile.readLine();
      } catch (IOException e) {
        System.err.println("Error reading from: " + inputFile);
        loaded = false;
      }
    } catch (FileNotFoundException e) {
      System.err.println("Error opening: " + inputFile);
      loaded = false;
    }
  }

  public String getMessage()
  {
    return log_message;
  }

  public String getNextMessage()
  {
    try {
      log_message = inFile.readLine();
    } catch (IOException e) {
      System.err.println("Error reading from: " + filename);
      loaded = false;
    }
    return log_message;
  }

  public String getNextUniqueMessage()
  {
    String current_message = "";
    boolean done = false;

    try {
      while (!done && inFile.ready()) {
        current_message = inFile.readLine();
        if (log_message.compareTo(current_message) != 0 && isValid(current_message)) {
          log_message = current_message;
          done = true;
        }
      }
    } catch (IOException e) {
      System.err.println("Error reading from: " + filename);
      loaded = false;
    }

    return log_message;
  }

  public boolean isValid(String message)
  {
    boolean valid = false;
    if (message.compareTo("") != 0) {
      String token = "";
      StringTokenizer tokens = new StringTokenizer(message, "() ");
      token = tokens.nextToken();
      if (token.compareTo("see") == 0) {
        valid = true;
      }
    }
    return valid;
  }
  
  public boolean isReady()
  {
    boolean ready = loaded;
    if (loaded) {
      try {
        ready = inFile.ready();
      }
      catch (IOException e) {
        System.err.println("Error:  File " + filename + " not loaded for reading");
      }
    }
    return ready;
  }

  // ******** END OF ADDITIONS ******** //

  // Sets the current time value
  public void setTime(float t)
  {
  time = (int) t;
  }

  // puts visual data in log
  public void visualData(ObjectInfo obj)
  {
    try {
      // format: (see TIME DISTANCE DIRECTION DISTCHNG DIRCHNG BodyDir HeadDir)
      String out = " (see " + time + " " + obj.m_type + " " + obj.m_distance
      + " " + obj.m_direction + " " + obj.m_distChange + " " + obj.m_dirChange + " BodyDir HeadDir) ";
      output.write(out.getBytes());
      output.flush();
      //System.out.println("Visual data logged.");
    }
    catch (IOException e)
    {
        System.out.println("Error writing to file!");
    }
  }

  // puts a simple(dash/turn/kick) action in log
  public void simpleAction(Command action)
  {  // format: (ACTION PARAMETER(s), @time TIME)
    String out;
    if (action.getVar2() == -999){ //no second variable
      out = " (" + action.getType() + " " + action.getVar1()
      + ", @time " + time + ") ";
    } else {
      out = " (" + action.getType() + " " + action.getVar1()
      + " " + action.getVar2() + ", @time " + time + ") ";
    }
    
    try {
      output.write(out.getBytes());
      output.flush();
      //System.out.println("Simple action logged.");
    }
    catch (IOException e)
    {
        System.out.println("Error writing to file!");
    }
  }

  /* To be implemented...

  public void audioData(....)
  {
  }

  public void complexAction(....)
  {
  }

  public void situation(....)
  {
  }

  */
}
