//
//	File:			Memory.java
//	Author:		Krzysztof Langner
//	Date:			1997/04/28

//  Modified by:  Paul Marlow, Amir Ghavam, Yoga Selvaraj
//  Course:       Software Agents
//  Date Due:     November 30, 2000   

//  Modified by:  Tarek Hassan
//  Date:         6 July 2001

import java.io.*;

class Memory
{
	volatile private VisualInfo	m_info;	// place where all visual information is stored
	volatile private SenseBodyInfo m_bodyInfo;	// place where all body information is stored
	final static int SIMULATOR_STEP = 100;
  private FileOutputStream output;

	//---------------------------------------------------------------------------
  // Default constructor, which just initializes an output file for use in
  // debugging.
	public Memory()
	{
    try
    {
      output = new FileOutputStream("memory.txt", true);
    }
    catch (FileNotFoundException e)
    {
      System.out.println("Error, file not found");
    }
	}

	//---------------------------------------------------------------------------
	// This function puts see information into our memory
	public void store(VisualInfo info)
	{
		m_info = info;
	}

	//---------------------------------------------------------------------------
	// This function puts sense body information into our memory
	public void store(SenseBodyInfo info)
	{
		m_bodyInfo = info;
    // printBodyInfo(); // used for printing sense_body messages to file
	}

	//---------------------------------------------------------------------------
	// This function looks for specified object
	public ObjectInfo getObject(String name)
	{
		if( m_info == null )
			waitForNewInfo();

		for(int c = 0 ; c < m_info.m_objects.size() ; c ++)
		{
			ObjectInfo object = (ObjectInfo)m_info.m_objects.elementAt(c);
			if(object.m_type.compareTo(name) == 0)
				return object;
		}

		return null;  // object was not found
	}

	//---------------------------------------------------------------------------
  // Prints a specified object to file - for testing and analysis
  public void printObject(ObjectInfo objInfo)
  {
    String objectType;

    try
    {
      if (objInfo != null)
        objectType = "{ " + objInfo.m_type +
          //" Dis: " + objInfo.m_distance + " DisChng: " + objInfo.m_distChange +
          " Dir: " + objInfo.m_direction + " DirChng: " + objInfo.m_dirChange +
          " }\n";
      else
        objectType = "{ NONE }\n";

      output.write(objectType.getBytes());
      output.flush();
    }
    catch (IOException e)
    {
        System.out.println("Error writing to file!");
    }
  }

	//---------------------------------------------------------------------------
  // Used for writing sense body information to a file
  public void printBodyInfo()
  {
    String bodyInfo = "(SENSE_BODY stamina: " + m_bodyInfo.stamina +
          " effort: " + m_bodyInfo.effort + " speed: " + m_bodyInfo.speed +
          " head_angle: " + m_bodyInfo.head_angle + ")\n";
    try
    {
      output.write(bodyInfo.getBytes());
      output.flush();
    }
    catch (IOException e)
    {
        System.out.println("Error writing to file!");
    }
  }

	//---------------------------------------------------------------------------
	// This function waits for new visual information
	public void waitForNewInfo()
	{
		// first remove old info
		m_info = null;
		// now wait until we get new copy
		while(m_info == null)
		{
			// We can get information faster then 75 miliseconds
			try
			{
				Thread.sleep(SIMULATOR_STEP);
			}
			catch(Exception e)
			{
			}
		}
	}
}

