//
//	File:			ObjectInfo.java
//	Author:		Krzysztof Langner
//	Date:			1997/04/28

//  Modified by:  Paul Marlow, Amir Ghavam, Yoga Selvaraj
//  Course:       Software Agents
//  Date Due:     November 30, 2000

import java.awt.Color;

//***************************************************************************
//
//	This is base class for different classese with visual information
//	about objects
//
//***************************************************************************
class ObjectInfo
{
  public String m_type;
  public float m_distance;
  public float m_direction;
  public float m_distChange;
  public float m_dirChange;
  public int x, y;
  protected boolean selected = false;
  protected Color defaultColor = Color.white;
  protected Color drawColor = defaultColor;

  //===========================================================================
  // Initialization member functions
  public ObjectInfo(String type)
  {
    m_type = type;
  }

  public void select(int mouse_x, int mouse_y)
  {
    selected = (x == mouse_x && y == mouse_y);
    if (selected) {
      drawColor = SoccerParams.selectedColor;
    }
    else {
      drawColor = defaultColor;
    }
  }

  public boolean isSelected()
  {
    return selected;
  }

  public Color getDrawColor()
  {
    return drawColor;
  }

  public void setDrawColor(Color new_color)
  {
    defaultColor = new_color;
    if (selected) {
      drawColor = SoccerParams.selectedColor;
    }
    else {
      drawColor = defaultColor;
    }
  }
}


//***************************************************************************
//
//	This class holds visual information about player
//
//***************************************************************************
class PlayerInfo extends ObjectInfo
{
  String  m_teamName = "";
  int m_uniformName = 0;        // recognise 0 as not being able to see number
  float m_bodyDir;
  float m_headDir;

  //===========================================================================
  // Initialization member functions
  public PlayerInfo()
  {
    super("player");
    setDrawColor(SoccerParams.unknownplayercolor);
  }

  public PlayerInfo(String team, int number)
  {
    super("player");
    m_teamName = team;
    m_uniformName = number;
    m_bodyDir = 0;
    m_headDir = 0;
    setDrawColor(SoccerParams.unknownplayercolor);
  }

  public PlayerInfo(String team, int number, float bodyDir, float headDir)
  {
    super("player");
    m_teamName = team;
    m_uniformName = number;
    m_bodyDir = bodyDir;
    m_headDir = headDir;
    setDrawColor(SoccerParams.unknownplayercolor);
  }

  public void select(int mouse_x, int mouse_y)
  {
    int end_x = x + SoccerParams.player_width;
    int end_y = y + SoccerParams.player_width;
    selected = ((x <= mouse_x && mouse_x <= end_x) && (y <= mouse_y && mouse_y <= end_y));
    if (selected) {
      drawColor = SoccerParams.selectedColor;
    }
    else {
      drawColor = defaultColor;
    }
  }
}


//***************************************************************************
//
//	This class holds visual information about goal
//
//***************************************************************************
class GoalInfo extends ObjectInfo
{
  //===========================================================================
  // Initialization member functions
  public GoalInfo()
  {
    super("goal");
    setDrawColor(SoccerParams.goalcolor);
  }

  public GoalInfo(char side)
  {
    super("goal " + side);
    setDrawColor(SoccerParams.goalcolor);
  }

  public void select(int mouse_x, int mouse_y)
  {
    int end_x = x + SoccerParams.goalpost_width;
    int end_y = y + SoccerParams.goalpost_width;
    selected = ((x <= mouse_x && mouse_x <= end_x) && (y <= mouse_y && mouse_y <= end_y));
    if (selected) {
      drawColor = SoccerParams.selectedColor;
    }
    else {
      drawColor = defaultColor;
    }
  }
}


//***************************************************************************
//
//	This class holds visual information about ball
//
//***************************************************************************
class BallInfo extends ObjectInfo
{
  //===========================================================================
  // Initialization member functions
  public BallInfo()
  {
    super("ball");
    setDrawColor(SoccerParams.ballcolor);
  }

  public void select(int mouse_x, int mouse_y)
  {
    int end_x = x + SoccerParams.ball_width;
    int end_y = y + SoccerParams.ball_width;
    selected = ((x <= mouse_x && mouse_x <= end_x) && (y <= mouse_y && mouse_y <= end_y));
    if (selected) {
      drawColor = SoccerParams.selectedColor;
    }
    else {
      drawColor = defaultColor;
    }
  }
}


//***************************************************************************
//
//	This class holds visual information about flag
//
//***************************************************************************
class FlagInfo extends ObjectInfo
{
  char m_type;  // p|g
  char m_pos1;  // t|b|l|c|r
  char m_pos2;  // l|r|t|c|b
  int m_num;    // 0|10|20|30|40|50
  boolean m_out;

  //===========================================================================
  // Initialization member functions
  public FlagInfo()
  {
    super("flag");
    m_type = ' ';
    m_pos1 = ' ';
    m_pos2 = ' ';
    m_num = 0;
    m_out = false;
    setDrawColor(SoccerParams.flagcolor);
  }

  public FlagInfo(String flagType, char type, char pos1, char pos2,
                  int num, boolean out)
  {
    super(flagType);
    m_type = type;
    m_pos1 = pos1;
    m_pos2 = pos2;
    m_num = num;
    m_out = out;
    setDrawColor(SoccerParams.flagcolor);
  }

  public FlagInfo(char type, char pos1, char pos2, int num, boolean out)
  {
    super("flag");
    m_type = type;
    m_pos1 = pos1;
    m_pos2 = pos2;
    m_num = num;
    m_out = out;
    setDrawColor(SoccerParams.flagcolor);
  }

  public void select(int mouse_x, int mouse_y)
  {
    int end_x = x + SoccerParams.flag_width;
    int end_y = y + SoccerParams.flag_width;
    selected = ((x <= mouse_x && mouse_x <= end_x) && (y <= mouse_y && mouse_y <= end_y));
    if (selected) {
      drawColor = SoccerParams.selectedColor;
    }
    else {
      drawColor = defaultColor;
    }
  }
}


//***************************************************************************
//
//	This class holds visual information about line
//
//***************************************************************************
class LineInfo extends ObjectInfo
{
  char m_kind;  // l|r|t|b

  //===========================================================================
  // Initialization member functions
  public LineInfo()
  {
    super("line");
    setDrawColor(SoccerParams.linecolor);
  }

  public LineInfo(char kind)
  {
    super("line");
    m_kind = kind;
    setDrawColor(SoccerParams.linecolor);
  }

  public void select(int mouse_x, int mouse_y)
  {

  }
}


