
//      File:   PlayerCanvas.java
//      Author: Tarek Hassan
//      Date:   22 June 2001
//

import java.awt.*;
import java.util.*;

public class PlayerCanvas extends Canvas {

  private Vector visuals;
  private Vector field_flags;
  private Vector selected;
  private int width = 100;
  private int height = 100;
  private int centre_x;
  private int centre_y;
  private Polygon field;
  private double perspective_width = 20;
  private String team = "";
  private char side;
  private int user_x ;//= 250;
  private int user_y ;//= 400;
  private int scale = 6;
  private int visible_dist = scale*3; //assuming server default of 3
  //public String statusLabel = new String("...");
  public InputHandler control;
  private PlayerFrame p_frame;

  public PlayerCanvas(PlayerFrame playerframe) {
    super();
    p_frame = playerframe;
    visuals = null;
    selected = new Vector();
    this.setBackground(SoccerParams.fieldcolor);
    this.setForeground(Color.white);
    control = new InputHandler(this);
    this.addMouseListener(control);
    this.addMouseMotionListener(control);
    //control.setCanvas(this);
  }

  public void paint(Graphics g)
  {   
    if (visuals != null) {
      setField(visuals);
      //System.out.println("** " + field_flags.size() + " field flags **");
      //drawField(field_flags, g);  // now just draws kickoff circle
    }
    // Draw user & vision cone
    // Note: if we allow user to alter their view_width, this will need to be
    // changed to reflect actual view instead of the default 90deg
    user_x = this.getWidth()/2;
    user_y = this.getHeight() - 65;

    g.setColor(SoccerParams.conecolor);
    g.drawLine(user_x, user_y, user_x - 500, user_y - 500);  // assuming 90deg
    g.drawLine(user_x, user_y, user_x + 500, user_y - 500);  // assuming 90deg
    g.drawOval(user_x - (visible_dist/2), user_y - (visible_dist/2), visible_dist, visible_dist);
    //g.setColor(conecolor.darker());
    //g.fillOval(user_x - ((visible_dist-2)/2), user_y - ((visible_dist-2)/2), (visible_dist)-2, (visible_dist)-2);
    g.setColor(SoccerParams.teamcolor);
    g.fillOval(user_x - (SoccerParams.player_width/2), user_y - (SoccerParams.player_width/2), SoccerParams.player_width, SoccerParams.player_width);

    if (visuals != null) {
      ObjectInfo obj;
      //System.out.println("Visuals size: " + visuals.size());
      for (int i = 0; i < visuals.size(); i++) {
        obj = (ObjectInfo) (visuals.elementAt(i));
        if (obj.m_type.compareToIgnoreCase("player") == 0) {
          drawPlayer((PlayerInfo)(obj), g);
        } else if (obj.m_type.compareToIgnoreCase("ball") == 0) {
          drawBall((BallInfo)(obj), g);
        } else if ((obj.m_type.compareTo("flag g l t") == 0) || (obj.m_type.compareTo("flag g l b") == 0) ||
          (obj.m_type.compareTo("flag g r t") == 0) || (obj.m_type.compareTo("flag g r b") == 0)){
          drawGoalPost((FlagInfo)(obj), g);
        } else if ((obj.m_type.compareTo("line") == 0)){
          drawFieldLine((LineInfo)(obj), g);
        } else if (!(obj.m_type.compareTo("goal l") == 0) && !(obj.m_type.compareTo("goal r") == 0)) {
          drawFlag((FlagInfo)(obj), g);
        }
      }
    } else {
      //System.out.println("Null");
    }
  }

  public void setObjects(Vector vobj)
  {
    visuals = vobj;
  }

  // puts all flags not 'out' of field into the Vector field_flags
  private void setField(Vector m_objects)
  {
    field_flags = new Vector(18);
    FlagInfo flag;
      for (int i = 0; i < m_objects.size(); i++){
        if (m_objects.elementAt(i) instanceof FlagInfo){
          flag = (FlagInfo)(m_objects.elementAt(i));
          //System.out.println(flag.m_out);
          if (!flag.m_out){ 
            field_flags.addElement(flag);
            //System.out.println(flag.m_type + "********");
          }
        }     
    //System.out.print("** " + field_flags.size() + " field flags **");
    //System.out.println("|||||| " + field_flags.toString() + " ||||||");
      }
  }

  public void setSquare(int w, int h)
  {
    width = w;
    height = h;
  }

  public void setTeam(String teamname)
  {
    team = teamname;
  }

  public void setSide(char s)
  {
    side = s;
  }

  private void drawGoalPost(FlagInfo goalpost, Graphics g)
  {
    int x, y;
    Point coordinate = calculatePoint(goalpost.m_distance, goalpost.m_direction, SoccerParams.goalpost_width);
    g.setColor(SoccerParams.goalcolor);
    g.fillRect(coordinate.x, coordinate.y, SoccerParams.goalpost_width, SoccerParams.goalpost_width);
    if (goalpost.m_pos1 == side){
      goalpost.setDrawColor(SoccerParams.teamcolor);
    } else {
      goalpost.setDrawColor(SoccerParams.opponentcolor);
    }
    //g.setColor(goalpost.getDrawColor());
    g.fillRect(coordinate.x+1, coordinate.y+1, SoccerParams.goalpost_width-2, SoccerParams.goalpost_width-2);
    goalpost.x = coordinate.x;
    goalpost.y = coordinate.y;
  }

  private void drawFlag(FlagInfo flag, Graphics g)
  {
    int x, y;
    Point coordinate = calculatePoint(flag.m_distance, flag.m_direction, SoccerParams.flag_width);
    //if (flag.m_out) {
    //  g.setColor(flagcolor);
    //} else {
    //  g.setColor(linecolor);
    //}
    g.setColor(flag.getDrawColor());
    g.fillRect(coordinate.x, coordinate.y, SoccerParams.flag_width, SoccerParams.flag_width);
    flag.x = coordinate.x;
    flag.y = coordinate.y;
  }

  //plots field lines between flags and the kickoff circle
  private void drawField(Vector m_flags, Graphics g)
  {
    ObjectInfo flag_a;
    ObjectInfo flag_b;
    g.setColor(SoccerParams.linecolor);
    if (m_flags != null){
      for (int i = 0; i < m_flags.size(); i++){
        flag_a = (ObjectInfo)(m_flags.elementAt(i));
       // centre circle
        if (flag_a.m_type.compareTo("flag c") == 0){
          int x, y;
          Point coordinate = calculatePoint(flag_a.m_distance, flag_a.m_direction, SoccerParams.centrecircle_width);
          g.drawOval(coordinate.x, coordinate.y, SoccerParams.centrecircle_width, SoccerParams.centrecircle_width);
        }
      }
    }
  }

  private void drawLine(ObjectInfo flag_a, ObjectInfo flag_b, Graphics g)
  {
    Point coordinate_a = calculatePoint(flag_a.m_distance, flag_a.m_direction, SoccerParams.flag_width);
    Point coordinate_b = calculatePoint(flag_b.m_distance, flag_b.m_direction, SoccerParams.flag_width);
    //g.setColor(flagcolor);        //set once in draw field
    g.drawLine(coordinate_a.x, coordinate_a.y, coordinate_b.x, coordinate_b.y);
  }

  private void drawFieldLine(LineInfo line, Graphics g)
  {
    // System.out.println(field_flags.size()+" field flags *****");
    ObjectInfo flag_a;
    ObjectInfo flag_b;
    g.setColor(line.getDrawColor());
    char ch;

    ch = 't';
    if ((line.m_kind == ch) && (field_flags != null)){
      for (int i = 0; i < field_flags.size(); i++){
        flag_a = (ObjectInfo)(field_flags.elementAt(i));
        if (flag_a.m_type.compareTo("flag r t") == 0){
          for (int j = 0; j < field_flags.size(); j++){
            flag_b = (ObjectInfo)(field_flags.elementAt(j));
            if (flag_b.m_type.compareTo("flag l t") == 0){
              drawLine(flag_a, flag_b, g);
              //drawLine(flag_a, perpVCP(line, flag_a, true), g);
              //drawLine(flag_b, perpVCP(line, flag_b, false), g);
              return;
            }
          }

          flag_b = vcPoint(line, false);
          drawLine(flag_a, flag_b, g);
          //drawLine(flag_a, perpVCP(line, flag_a, true), g);
          return;
        } else if (flag_a.m_type.compareTo("flag l t") == 0){
          for (int j = 0; j < field_flags.size(); j++){
            flag_b = (ObjectInfo)(field_flags.elementAt(j));
            if (flag_b.m_type.compareTo("flag r t") == 0){
              drawLine(flag_a, flag_b, g);
              //drawLine(flag_a, perpVCP(line, flag_a, false), g);
              //drawLine(flag_b, perpVCP(line, flag_b, true), g);
              return;
            }
          }

          flag_b = vcPoint(line, true);
          drawLine(flag_a, flag_b, g);
          //drawLine(flag_a, perpVCP(line, flag_a, false), g);
          return;
        }
      }
      // only do if all flags have been checked
      flag_a = vcPoint(line, true);
      flag_b = vcPoint(line, false);
      drawLine(flag_a, flag_b, g);
      return;
    }

    ch = 'b';
    if ((line.m_kind == ch) && (field_flags != null)){
      for (int i = 0; i < field_flags.size(); i++){
        flag_a = (ObjectInfo)(field_flags.elementAt(i));
        if (flag_a.m_type.compareTo("flag r b") == 0){
          for (int j = 0; j < field_flags.size(); j++){
            flag_b = (ObjectInfo)(field_flags.elementAt(j));
            if (flag_b.m_type.compareTo("flag l b") == 0){
              drawLine(flag_a, flag_b, g);
              //drawLine(flag_a, perpVCP(line, flag_a, false), g);
              //drawLine(flag_b, perpVCP(line, flag_b, true), g);
              return;
            }
          }

          flag_b = vcPoint(line, true);
          drawLine(flag_a, flag_b, g);
          //drawLine(flag_a, perpVCP(line, flag_a, false), g);
          return;
        } else if (flag_a.m_type.compareTo("flag l b") == 0){
          for (int j = 0; j < field_flags.size(); j++){
            flag_b = (ObjectInfo)(field_flags.elementAt(j));
            if (flag_b.m_type.compareTo("flag r b") == 0){
              drawLine(flag_a, flag_b, g);
              //drawLine(flag_a, perpVCP(line, flag_a, false), g);
              //drawLine(flag_b, perpVCP(line, flag_b, true), g);
              return;
            }
          }

          flag_b = vcPoint(line, false);
          drawLine(flag_a, flag_b, g);
          //drawLine(flag_a, perpVCP(line, flag_a, true), g);
          return;
        }
      }
      // only do if all flags have been checked
      flag_a = vcPoint(line, true);
      flag_b = vcPoint(line, false);
      drawLine(flag_a, flag_b, g);
      return;
    }

    ch = 'r';
    if ((line.m_kind == ch) && (field_flags != null)){
      for (int i = 0; i < field_flags.size(); i++){
        flag_a = (ObjectInfo)(field_flags.elementAt(i));
        if (flag_a.m_type.compareTo("flag r b") == 0){
          for (int j = 0; j < field_flags.size(); j++){
            flag_b = (ObjectInfo)(field_flags.elementAt(j));
            if (flag_b.m_type.compareTo("flag r t") == 0){
              drawLine(flag_a, flag_b, g);
              //drawLine(flag_a, perpVCP(line, flag_a, true), g);
              //drawLine(flag_b, perpVCP(line, flag_b, false), g);
              return;
            }
          }
          flag_b = vcPoint(line, false);
          drawLine(flag_a, flag_b, g);
          //drawLine(flag_a, perpVCP(line, flag_a, true), g);
          return;
        } else if (flag_a.m_type.compareTo("flag r t") == 0){
          for (int j = 0; j < field_flags.size(); j++){
            flag_b = (ObjectInfo)(field_flags.elementAt(j));
            if (flag_b.m_type.compareTo("flag r b") == 0){
              drawLine(flag_a, flag_b, g);
              //drawLine(flag_a, perpVCP(line, flag_a, true), g);
              //drawLine(flag_b, perpVCP(line, flag_b, false), g);
              return;
            }
          }
          flag_b = vcPoint(line, true);
          drawLine(flag_a, flag_b, g);
          //drawLine(flag_a, perpVCP(line, flag_a, false), g);
          return;
        }
      }
      // only do if all flags have been checked
      flag_a = vcPoint(line, true);
      flag_b = vcPoint(line, false);
      drawLine(flag_a, flag_b, g);
      return;
    }

    ch = 'l';
    if ((line.m_kind == ch) && (field_flags != null)){
      for (int i = 0; i < field_flags.size(); i++){
        flag_a = (ObjectInfo)(field_flags.elementAt(i));
        if (flag_a.m_type.compareTo("flag l b") == 0){
          for (int j = 0; j < field_flags.size(); j++){
            flag_b = (ObjectInfo)(field_flags.elementAt(j));
            if (flag_b.m_type.compareTo("flag l t") == 0){
              drawLine(flag_a, flag_b, g);
              //drawLine(flag_a, perpVCP(line, flag_a, false), g);
              //drawLine(flag_b, perpVCP(line, flag_b, true), g);
              return;
            }
          }
          flag_b = vcPoint(line, true);
          drawLine(flag_a, flag_b, g);
          //drawLine(flag_a, perpVCP(line, flag_a, false), g);
          return;
        } else if (flag_a.m_type.compareTo("flag l t") == 0){
          for (int j = 0; j < field_flags.size(); j++){
            flag_b = (ObjectInfo)(field_flags.elementAt(j));
            if (flag_b.m_type.compareTo("flag l b") == 0){
              drawLine(flag_a, flag_b, g);
              //drawLine(flag_a, perpVCP(line, flag_a, false), g);
              //drawLine(flag_b, perpVCP(line, flag_b, true), g);
              return;
            }
          }
          flag_b = vcPoint(line, false);
          drawLine(flag_a, flag_b, g);
          //drawLine(flag_a, perpVCP(line, flag_a, true), g);
          return;
        }
      }
      // only do if all flags have been checked
      flag_a = vcPoint(line, true);
      flag_b = vcPoint(line, false);
      drawLine(flag_a, flag_b, g);
      return;
    }
  }

  private void drawBall(BallInfo ball, Graphics g)
  {
    int x, y;
    Point coordinate = calculatePoint(ball.m_distance, ball.m_direction, SoccerParams.ball_width);
    //ball outline
    g.setColor(Color.black);
    g.fillOval(coordinate.x, coordinate.y, (SoccerParams.ball_width + 2), (SoccerParams.ball_width + 2));
    g.setColor(ball.getDrawColor());
    g.fillOval(coordinate.x, coordinate.y, SoccerParams.ball_width, SoccerParams.ball_width);
    ball.x = coordinate.x;
    ball.y = coordinate.y;
  }

  private void drawPlayer(PlayerInfo player, Graphics g)
  {
    int x, y;

    if (player.m_teamName.compareTo(team) == 0) {
      player.setDrawColor(SoccerParams.teamcolor);
    } else if (player.m_teamName.compareTo(team) != 0 && player.m_teamName.compareTo("") != 0) {
      player.setDrawColor(SoccerParams.opponentcolor);
    } else {
      player.setDrawColor(SoccerParams.unknownplayercolor);
    }

    g.setColor(player.getDrawColor());
    Point coordinate = calculatePoint(player.m_distance, player.m_direction, SoccerParams.player_width);
    g.fillOval(coordinate.x, coordinate.y, SoccerParams.player_width, SoccerParams.player_width);
    g.setColor(SoccerParams.ballcolor);
    Integer number = new Integer(player.m_uniformName);
    g.drawString(number.toString(), coordinate.x, coordinate.y);
    player.x = coordinate.x;
    player.y = coordinate.y;
  }

  // returns the point where the line ends at the vision cone
  private ObjectInfo vcPoint(ObjectInfo line, boolean right)
  {
    double x;
    //double dir = Math.abs(line.m_direction);
    double dist = line.m_distance;
    double dir = (int)line.m_direction;
    //int dist = (int)line.m_distance;

    if (right){
      x = (dist * Math.sin(Math.toRadians(180)-Math.toRadians(dir))) /
      (Math.sin(Math.toRadians(dir)-Math.toRadians(45)));
    } else {
      x = (dist * Math.sin(Math.toRadians(dir))) /
      (Math.sin(Math.toRadians(135)-Math.toRadians(dir)));
    }
    ObjectInfo o = new ObjectInfo("dud");
    o.m_distance = (float) x;
    if (right){
      o.m_direction = 45;
    } else {
      o.m_direction = -45;
    }
    return o;
  }

  // returns the point where the perpendicular line ends at the vision cone
   private ObjectInfo perpVCP(ObjectInfo origLine, ObjectInfo cornerPoint, boolean right)
  {
    ObjectInfo perpLine = new ObjectInfo("dud");
    double lineangle = origLine.m_direction;

    // Set up for correct positive angle for calculations
    if (!right && lineangle < 0) {
      lineangle = Math.abs(lineangle);
    }
    else if (!right && lineangle >= 0) {
      lineangle = 180 - lineangle;
    }
    else if (right && lineangle < 0) {
      lineangle = 180 + lineangle;
    }
    else {
      // leave unchanged
    }

    double x =  Math.sin(Math.toRadians(Math.abs(cornerPoint.m_direction))) * (cornerPoint.m_distance / Math.sin(Math.toRadians(180 - lineangle)));
    double a = x * Math.sqrt(2);
    perpLine.m_distance = (float) (Math.sin(Math.toRadians(135 - lineangle)) *
        (a / Math.sin(Math.toRadians(45))));

    if (right) {
      perpLine.m_direction = -45;
    }
    else {
      perpLine.m_direction = 45;
    }

    return perpLine;

    /*
    int cpDist = (int) cornerPoint.m_distance;
    int oDir = (int) origLine.m_direction;
    //System.out.print("oDir = " + oDir + "   |   ");
    int cpDir = (int) cornerPoint.m_direction;
    ObjectInfo perpLine = new ObjectInfo("dud");

    if (oDir > 0){                       // ???
      perpLine.m_direction = oDir - 90;
    } else {
      perpLine.m_direction = oDir + 90;
    }
    //System.out.println("pDir = " + (int)perpLine.m_direction + "   *****");
    perpLine.m_distance = (float)((cpDist * Math.sin(Math.toRadians(oDir)
    - Math.toRadians(cpDir) - Math.toRadians(90)))
    / (Math.sin(Math.toRadians(90)- Math.toRadians(oDir))));
    return vcPoint(perpLine, right);
    */
  }

  // Calculates the coordinate of the object - it's from this that the actual
  // top/left coordinates are determined based on width, and height of object.
  private Point calculatePoint(double distance, double direction, int width)
  {        /*
    // Calculate the length down the field in pixels per unit of distance as well as
    // the width distance
    double pixel_length = ((this.getHeight() / 2) / SoccerParams.field_length);
    double pixel_width = ((this.getWidth() / 2) / SoccerParams.field_width);
    // Calculate distance straight down the field
    double relative_dis = distance * Math.cos(Math.toRadians(Math.abs(direction)));
    // Calcuate y coordinate
    int y = (int)(this.getHeight() - (relative_dis * pixel_length));

    // Calculate the distance width compared to the angle
    double relative_width = (distance * Math.sin(Math.toRadians(direction))) * pixel_width;
    // Calculate the x coordinate
    int x = (int)(relative_width + (this.getWidth() / 2));
            */
    int relative_x =  scale * (int) (distance * Math.sin(Math.toRadians(direction)));
    int relative_y =  scale * (int) (-1 * distance * Math.cos(Math.toRadians(direction)));

    int x = relative_x + user_x -(width/2);
    int y = relative_y + user_y -(width/2);

    return new Point(x, y);
  }

  public Point getUserCoordinates()
  {
    if (this != null)
    {
      return new Point(user_x, user_y);
    } else return null;
  }

  public void selectObject(int x, int y)
  {
    selected.clear();
    ObjectInfo obj = null;
    for (int i = 0; i < visuals.size(); i++) {
      obj = (ObjectInfo) (visuals.elementAt(i));
      obj.select(x, y);
      if (obj.isSelected()) {
        selected.addElement(obj);
      }
    }
    if (selected.size() == 0) {
      p_frame.initButtons();
      p_frame.resetLabels();
    }
    else {
      p_frame.directionList.setEnabled(true);
      p_frame.distanceList.setEnabled(true);
      for (int j = 0; j < selected.size(); j++) {
        obj = (ObjectInfo) (selected.elementAt(j));
        p_frame.btnNear.setEnabled(true);
        p_frame.btnFar.setEnabled(true);
        if (obj.m_type.compareTo("ball") == 0) {
          p_frame.btnKickable.setEnabled(true);
        }
        else {
          p_frame.btnKickable.setEnabled(false);
        }
        p_frame.setLabels(String.valueOf(obj.m_distance), String.valueOf(obj.m_direction));
      }
    }
    repaint();
  }

  public Vector getSelectedObjects()
  {
    return selected;
  }
}
