
//	File:			PlayerFrame.java
//	Author:		Tarek Hassan
//	Date:			10 July 2001
//

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.SwingConstants;
import javax.swing.border.*;
import java.awt.FileDialog;
import java.awt.Toolkit;
import java.util.StringTokenizer;
import java.util.Vector;

public class PlayerFrame extends JFrame {
  private float stamina = 2000;
  private float time = 0;
  private String m_team = "";
  private PlayerCanvas p_canvas;
  private Log log_file = null;
  private ARFFWriter out_position = null;

  BorderLayout b_layout = new BorderLayout();

  FileDialog openFile = null;

  JToolBar toolBar = new JToolBar(JToolBar.TOP);

  // ToolBar buttons
  JButton btnExit = null;
  JButton btnOpen = null;
  JButton btnKickable = null;
  JButton btnNear = null;
  JButton btnFar = null;
  JButton btnNext = null;
  JButton btnArff = null;
  JButton btnAdd = null;
  
  JComboBox distanceList = null;
  JComboBox directionList = null;  
  
  // Information Panel
  JPanel pnlInfo = new JPanel();
  JLabel lblDistance = null;
  JLabel lblDirection = null;

  //Construct the frame
  public PlayerFrame() {
    ARFFInit();
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    p_canvas = new PlayerCanvas(this);
    //this.addKeyListener(p_canvas.control);
    try  {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  //ARFF File Initialization
  private void ARFFInit()
  {
    out_position = new ARFFWriter("Object_Position");
    out_position.addAttribute("Distance", "real");
    out_position.addAttribute("Direction", "real");
    out_position.addAttribute("Position", "{ExtremeLeft_Close, ExtremeLeft_Near, ExtremeLeft_Far, ExtremeLeft_Distant, Left_Close, Left_Near, Left_Far, Left_Distant, Front_Close, Front_Near, Front_Far, Front_Distant, Right_Close, Right_Near, Right_Far, Right_Distant, ExtremeRight_Close, ExtremeRight_Near, ExtremeRight_Far, ExtremeRight_Distant}");
  }

  //Component initialization
  private void jbInit() throws Exception  {
    openFile = new FileDialog(this, "Select player log file to Open", FileDialog.LOAD);

    this.getContentPane().setLayout(b_layout);
    this.setSize(new Dimension(651, 542));
    this.setTitle("RIP: RoboCup Individual Player Log Classifier");

    // Get default ToolKit and Screen Size
    Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
    Dimension screensize = defaultToolkit.getScreenSize();

    // Set up ToolBar
    Insets margin = new Insets(1, 1, 1, 1);
    toolBar.setMargin(margin);

    // Create ToolBar Buttons

    // Exit Button
    btnExit = new JButton(new ImageIcon("ExitEnabled.gif"));
    btnExit.setDisabledIcon(new ImageIcon("ExitDisabled.gif"));
    btnExit.setMaximumSize(new Dimension(24, 24));
    btnExit.setMinimumSize(new Dimension(24, 24));
    btnExit.setToolTipText("Exit");
    btnExit.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        btnExit_actionPerformed(e);
      }
    });

    // Open Button
    btnOpen = new JButton(new ImageIcon("OpenEnabled.gif"));
    btnOpen.setDisabledIcon(new ImageIcon("OpenDisabled.gif"));
    btnOpen.setMaximumSize(new Dimension(24, 24));
    btnOpen.setMinimumSize(new Dimension(24, 24));
    btnOpen.setToolTipText("Open Log File");
    btnOpen.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        btnOpen_actionPerformed(e);
      }
    });

    // Kickable Button
    btnKickable = new JButton(new ImageIcon("KickableEnabled.gif"));
    btnKickable.setDisabledIcon(new ImageIcon("KickableDisabled.gif"));
    btnKickable.setMaximumSize(new Dimension(24, 24));
    btnKickable.setMinimumSize(new Dimension(24, 24));
    btnKickable.setToolTipText("Kickable");
    btnKickable.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        btnKickable_actionPerformed(e);
      }
    });

    // Near Button
    btnNear = new JButton(new ImageIcon("NearEnabled.gif"));
    btnNear.setDisabledIcon(new ImageIcon("NearDisabled.gif"));
    btnNear.setMaximumSize(new Dimension(24, 24));
    btnNear.setMinimumSize(new Dimension(24, 24));
    btnNear.setToolTipText("Near");
    btnNear.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        btnNear_actionPerformed(e);
      }
    });

    // Far Button
    btnFar = new JButton(new ImageIcon("FarEnabled.gif"));
    btnFar.setDisabledIcon(new ImageIcon("FarDisabled.gif"));
    btnFar.setMaximumSize(new Dimension(24, 24));
    btnFar.setMinimumSize(new Dimension(24, 24));
    btnFar.setToolTipText("Far");
    btnFar.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        btnFar_actionPerformed(e);
      }
    });

    // Next Button
    btnNext = new JButton(new ImageIcon("NextEnabled.gif"));
    btnNext.setDisabledIcon(new ImageIcon("NextDisabled.gif"));
    btnNext.setMaximumSize(new Dimension(24, 24));
    btnNext.setMinimumSize(new Dimension(24, 24));
    btnNext.setToolTipText("Next Log Record");
    btnNext.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        btnNext_actionPerformed(e);
      }
    });

    // Generate ARFF Button
    btnArff = new JButton("Generate");
    //btnArff = new JButton(new ImageIcon("ArffEnabled.gif"));
    //btnArff.setDisabledIcon(new ImageIcon("ArffDisabled.gif"));
    //btnArff.setMaximumSize(new Dimension(24, 24));
    //btnArff.setMinimumSize(new Dimension(24, 24));
    btnArff.setToolTipText("Generate ARFF");
    btnArff.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        btnArff_actionPerformed(e);
      }
    });

    // Generate ARFF Button
    btnAdd = new JButton("Add");
    //btnAdd = new JButton(new ImageIcon("AddEnabled.gif"));
    //btnAdd.setDisabledIcon(new ImageIcon("AddDisabled.gif"));
    //btnAdd.setMaximumSize(new Dimension(24, 24));
    //btnAdd.setMinimumSize(new Dimension(24, 24));
    btnAdd.setToolTipText("Add to ARFF");
    btnAdd.addActionListener(new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        btnAdd_actionPerformed(e);
      }
    });

    String[] distances = {"Select Distance", "Close", "Near", "Far", "Distant"};
    String[] directions = {"Select Direction", "ExtremeLeft", "Left", "Front", "Right", "ExtremeRight"};
    
    distanceList = new JComboBox(distances);
    distanceList.setSelectedIndex(0);
    distanceList.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        distanceList_actionPerformed(e);
      }
    });
    
    directionList = new JComboBox(directions);
    directionList.setSelectedIndex(0);
    directionList.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        directionList_actionPerformed(e);
      }
    });

    // Initialize button enabling/disabling
    initButtons();

    // Add Buttons to the ToolBar
    toolBar.setOrientation(toolBar.VERTICAL);
    toolBar.add(btnExit);
    toolBar.add(btnOpen);
    toolBar.addSeparator(new Dimension(4, 4));
    toolBar.add(btnKickable);
    toolBar.addSeparator(new Dimension(4, 4));
    //toolBar.add(btnNear);
    //toolBar.add(btnFar);
    //toolBar.addSeparator(new Dimension(4, 4));
    toolBar.add(btnNext);
    toolBar.addSeparator(new Dimension(4, 4));
    toolBar.add(directionList);
    toolBar.addSeparator(new Dimension(4, 4));
    toolBar.add(distanceList);        
    toolBar.addSeparator(new Dimension(4, 4));
    toolBar.add(btnAdd);
    toolBar.addSeparator(new Dimension(4, 4));
    toolBar.add(btnArff);

    // Create information panel
    lblDistance = new JLabel("");
    lblDistance.setSize(new Dimension(50, 20));
    lblDirection = new JLabel("");
    lblDirection.setSize(new Dimension(50, 20));
    pnlInfo.setLayout(new BorderLayout());
    pnlInfo.setMinimumSize(new Dimension(200, 24));
    pnlInfo.setMaximumSize(new Dimension((int) (screensize.getWidth()), 24));
    pnlInfo.add(lblDistance, BorderLayout.WEST);
    pnlInfo.add(lblDirection, BorderLayout.EAST);

    this.getContentPane().add(toolBar, BorderLayout.EAST);
    this.getContentPane().add(pnlInfo, BorderLayout.SOUTH);
    this.getContentPane().add(p_canvas, BorderLayout.CENTER);
    p_canvas.repaint();
  }

  // Button Enabling/Disabling
  public void initButtons()
  {
    boolean valid;
    if (log_file != null) {
      valid = log_file.isReady();
    }
    else {
      valid = false;
    }

    btnNext.setEnabled(valid);
    btnExit.setEnabled(true);
    btnOpen.setEnabled(true);
    btnKickable.setEnabled(false);
    btnNear.setEnabled(false);
    btnFar.setEnabled(false);
    btnNear.setVisible(false);
    btnFar.setVisible(false);
    btnArff.setEnabled((out_position.getDataSize() > 0));
    btnAdd.setEnabled(false);
    directionList.setEnabled(false);
    distanceList.setEnabled(false);
  }

  public void setLabels(String distance, String direction)
  {
    lblDistance.setText("Distance: " + distance);
    lblDirection.setText("Direction: " + direction);
  }

  public void resetLabels()
  {
    lblDistance.setText("");
    lblDirection.setText("");
  }

  public void btnOpen_actionPerformed(ActionEvent e) {
    openFile.show();
    if (openFile.getFile() != null)
    {
      String path = openFile.getDirectory() + openFile.getFile();
      log_file = new Log(path);
      initButtons();
      // Client side init command
      parseLogMessage(log_file.getMessage());
      // Server side init response
      parseLogMessage(log_file.getNextUniqueMessage());
      // First sensory information record
      parseLogMessage(log_file.getNextUniqueMessage());
    }
  }

  public void btnExit_actionPerformed(ActionEvent e) {
    System.exit(0);
  }

  public void btnHelp_actionPerformed(ActionEvent e) {

  }

  void btnKickable_actionPerformed(ActionEvent e) {

  }

  void btnNear_actionPerformed(ActionEvent e) {
    Vector selected = p_canvas.getSelectedObjects();
    String data = "";
    ObjectInfo obj = null;

    for (int i = 0; i < selected.size(); i++) {
      obj = (ObjectInfo)(selected.elementAt(i));
      data = (new Float(obj.m_distance)).toString() + ",near";
      out_position.addData(data);
    }
  }

  void btnFar_actionPerformed(ActionEvent e) {
    Vector selected = p_canvas.getSelectedObjects();
    String data = "";
    ObjectInfo obj = null;

    for (int i = 0; i < selected.size(); i++) {
      obj = (ObjectInfo)(selected.elementAt(i));
      data = (new Float(obj.m_distance)).toString() + ",far";
      out_position.addData(data);
    }
  }

  void btnNext_actionPerformed(ActionEvent e) {
    initButtons();
    resetLabels();
    parseLogMessage(log_file.getNextUniqueMessage());
  }

  void btnArff_actionPerformed(ActionEvent e) {
    initButtons();
    resetLabels();
    out_position.buildARFF();
  }

  void btnAdd_actionPerformed(ActionEvent e) {
    Vector selected = p_canvas.getSelectedObjects();
    String data = "";
    ObjectInfo obj = null;

    for (int i = 0; i < selected.size(); i++) {
      obj = (ObjectInfo)(selected.elementAt(i));
      data = (new Float(obj.m_distance)).toString() + "," + (new Float(obj.m_direction)).toString() + "," +
            (String)directionList.getSelectedItem() + "_" + (String)distanceList.getSelectedItem();
      out_position.addData(data);
    }
    initButtons();
    resetLabels();
  }

  void distanceList_actionPerformed(ActionEvent e) {
    if (distanceList.getSelectedIndex() != 0 && directionList.getSelectedIndex() != 0) {
      btnAdd.setEnabled(true);
    }
    else {
      btnAdd.setEnabled(false);
    }
  }
  
  void directionList_actionPerformed(ActionEvent e) {
    if (distanceList.getSelectedIndex() != 0 && directionList.getSelectedIndex() != 0) {
      btnAdd.setEnabled(true);
    }
    else {
      btnAdd.setEnabled(false);
    }
  }

  //Overridden so we can exit on System Close
  protected void processWindowEvent(WindowEvent e) {
    super.processWindowEvent(e);
    if(e.getID() == WindowEvent.WINDOW_CLOSING) {
      btnExit_actionPerformed(null);
    }
  }

  public void setTeam(String team) {
    m_team = team;
  }

  //---------------------------------------------------------------------------
  // This function parses individual log messages
  private void parseLogMessage(String message)
  {
    if (log_file.isReady()) {
      // First check kind of information
      if(message.charAt(1) == 'i' ) // Init message
      {
        parseInitCommand(message);
      }
      else // See, Hear, or Sense_body command
      {
        parseSensorInformation(message);
      }
    }
  }

  //---------------------------------------------------------------------------
  // This function parses sensor information
  private void parseSensorInformation(String message)
  {
    if (log_file.isReady()) {
      // First check kind of information
      if( message.charAt(3) == 'e' ) // seE information
      {
        VisualInfo info = new VisualInfo(message);
        info.parse();
        p_canvas.setObjects(info.visuals);
        p_canvas.repaint();
      }
      else if (message.charAt(3) == 'n') // seNse information
      {
        //SenseBodyInfo senseInfo = new SenseBodyInfo(message);
        //stamina = senseInfo.stamina;
        //time = senseInfo.time;
        //p_canvas.repaint();
      }
      else if( message.charAt(3) == 'a' ) // heAr information
      {
        // parseHear(message);
      }
      else
      {
        //System.out.println("Error:  Format is not recognised \"" + message + "\"");
      }
    }
  }

  //---------------------------------------------------------------------------
  // This function parses initial message from the server
  protected void parseInitCommand(String message)
  {
    StringTokenizer tokenizer = new StringTokenizer(message,"() ");

    // We need init token
    if(tokenizer.nextToken().compareTo("init") != 0)
    {
      System.err.println("Error:  Incorrect init format.");
    }
    else {   
      // Read tokenized string
      String token = tokenizer.nextToken();

      // Check to see if it is a Client init command (i.e. teamname) or
      // a server init response (i.e. side - char)
      if (token.length() == 1) {
        // Server side init response
        char side = token.charAt(0);
        p_canvas.setSide(side);
        String playerNum = tokenizer.nextToken();
        String playMode = tokenizer.nextToken();
      }
      else {
        // Client side init command
        p_canvas.setTeam(token);
      }
    }
  }
}

